#setwd("C:/Users/ythuang/Desktop/mediation_test3/R codes")
load("mediation_test2.RData")
load("annot450KLong.RData")
library(limma)	# propTrueNull()
library(fdrtool)	# fdrtool()
source("MT_Comp.R")


####################################
## Step 1: S-M and M-Y assocition ##
####################################

# Running time: 108.67s
J<-dim(M.cand)[1]
a<-NULL; b<-NULL
for (j in 1:J){
	fit.m<-lm(M.cand[j,]~X+S)
	fit.y<-lm(Y~X+S+M.cand[j,])
	fit.m.sum<-summary(fit.m)
	fit.y.sum<-summary(fit.y)
	a.hat<-fit.m.sum$coef["S", 1]
	a.sd<-fit.m.sum$coef["S", 2]
	b.hat<-fit.y.sum$coef["M.cand[j, ]", 1]
	b.sd<-fit.y.sum$coef["M.cand[j, ]", 2]
	a<-c(a, a.hat/a.sd)
	b<-c(b, b.hat/b.sd)
	print(paste(j, "/", J)); flush.console()
}


#####################################
## Step 2: Perform mediation tests ##
#####################################

## Our proposed method: 14.16s 
pp.hat.comp<-MT_Comp(a, b)
pp.hat.comp[pp.hat.comp>1]<-1

## Joint significance test: 0.04s
pa<-2*pnorm(abs(a), low=F)
pb<-2*pnorm(abs(b), low=F)
pp.jt<-apply(cbind(pa, pb), 1, max)

## Normality-based test: 0.01s
pp.n<-2*pnorm(abs(a*b/sqrt(a^2+b^2)), low=F)

## Composite test estimating the proportion of null: 6.88s
ab<-a*b
pp0<-sapply(abs(ab)/sqrt(1), myp)
p1.hat<-propTrueNull(pa)
p2.hat<-propTrueNull(pb)
w0<-p1.hat*p2.hat; w1<-(1-p2.hat)*p1.hat; w2<-(1-p1.hat)*p2.hat
w<-w0+w1+w2; w0<-w0/w; w1<-w1/w; w2<-w2/w
pp.tilde.comp<-w0*pp0+w1*pa+w2*pb

## Bootstrap tests: 29.23 hours; read in the results
pp.b<-read.csv("Pvals_boot.csv", h=T, row.names=1)[,1]
pp.b[pp.b==0]<-0.5/1000

pp<-cbind(pp.hat.comp, pp.tilde.comp, pp.jt, pp.b, pp.n)


###############################
## Step 3: Summarize results ##
###############################

## Histogram

par(mfrow=c(3,2))
hist(pp.hat.comp, br=100)
hist(pp.tilde.comp, br=100)
hist(pp.jt, br=100)
hist(pp.b, br=100)
hist(pp.n, br=100)

## No (%) at cut-offs
apply(pp<0.1, 2, sum)
round(apply(pp<0.1, 2, mean)*100, 2)
apply(pp<0.05, 2, sum)
round(apply(pp<0.05, 2, mean)*100, 2)
apply(pp<0.01, 2, sum)
round(apply(pp<0.01, 2, mean)*100, 2)
apply(pp<0.005, 2, sum)
round(apply(pp<0.005, 2, mean)*100, 2)
apply(pp<0.001, 2, sum)
round(apply(pp<0.001, 2, mean)*100, 2)

## False discovary rate
fdr.hat.comp<-fdrtool(pp.hat.comp, statistic="pvalue")
fdr.tilde.comp<-fdrtool(pp.tilde.comp, statistic="pvalue")
fdr.jt<-fdrtool(pp.jt, statistic="pvalue")
fdr.b<-fdrtool(pp.b, statistic="pvalue")
fdr.n<-fdrtool(pp.n, statistic="pvalue")

table(fdr.hat.comp$qval<0.1)
table(fdr.tilde.comp$qval<0.1)
table(fdr.jt$qval<0.1)
table(fdr.b$qval<0.1)
table(fdr.n$qval<0.1)

annot<-annot450KLong
rownames(annot)<-annot450KLong$Name
annot<-annot[rownames(M.cand),]
annot[order(fdr.hat.comp$qval)[1:10], "UCSC_RefGene_Name"]
pp[annot$Name=="cg11950105",]


## QQ-plot
par(mfrow=c(2,2), mar=c(4,4,3,2))
set.seed(37)
pt.sort<-sort(runif(length(pp.jt)))
plot(y=-log10(sort(pp[,5])), x=-log10(pt.sort), col="gray20", pch=20, 
	xlim=c(0,5), ylim=c(0,5), xlab=expression(paste("Theoretical", -log[10]*italic(p))),
	ylab=expression(paste("Observed ", -log[10]*italic(p))), main=expression(list(italic(p[JT]), italic(p[N]))))
points(y=-log10(sort(pp[,3])), x=-log10(pt.sort), col="gray80", pch=20)
legend("topleft", legend=c("JT", "N"), pch=20, bty="n", col=c("gray80", "gray20"))
abline(0, 1)

plot(y=-log10(sort(pp[,4])), x=-log10(pt.sort), col="gray", pch=20, 
	xlim=c(0,5), ylim=c(0,5), xlab=expression(paste("Theoretical", -log[10]*italic(p))),
	ylab=expression(paste("Observed ", -log[10]*italic(p))), main=expression(italic(p[B])))
abline(0, 1)

plot(y=-log10(sort(pp[,2])), x=-log10(pt.sort), col="gray", pch=20, 
	xlim=c(0,5), ylim=c(0,5), xlab=expression(paste("Theoretical", -log[10]*italic(p))),
	ylab=expression(paste("Observed ", -log[10]*italic(p))), main=expression(italic(tilde(p)[comp])))
abline(0, 1)

plot(y=-log10(sort(pp[,1])), x=-log10(pt.sort), col="gray", pch=20, 
	xlim=c(0,5), ylim=c(0,5), xlab=expression(paste("Theoretical", -log[10]*italic(p))),
	ylab=expression(paste("Observed ", -log[10]*italic(p))), main=expression(italic(hat(p)[comp])))
abline(0, 1)



###############################################################################
## Appendix codes for bootstrap test: 64.18s*16394/10= 29.23hours
Pvals.boot<-NULL
system.time(
#for (j in which(annot$Name=="cg11950105")){	# require 10,000 replicates
for (j in 1:10){
	ab.hat.boot<-NULL
	M.cand.j<-as.numeric(M.cand[j,])
	for (b in 0:1000){
		set.seed(37+b)
		boot<-1:length(Y)
		if (b>0) boot<-sample(length(Y), re=T)
		M.cand.j.boot<-M.cand.j[boot]
		X.boot<-X[boot,]
		S.boot<-S[boot]
		Y.boot<-Y[boot]
		fit.m.boot<-lm(M.cand.j.boot~X.boot+S.boot)
		fit.y.boot<-lm(Y.boot~X.boot+S.boot+M.cand.j.boot)
		a.hat.boot<-fit.m.boot$coef["S.boot"]
		b.hat.boot<-fit.y.boot$coef["M.cand.j.boot"]
		ab.hat.boot<-c(ab.hat.boot, a.hat.boot*b.hat.boot)
		print(paste(j, ":", b, "/", J)); flush.console()
	}
	ab.hat.boot.0<-ab.hat.boot-mean(ab.hat.boot)
	pvals.boot=2*min(mean(ab.hat.boot>0), mean(ab.hat.boot<0))
	Pvals.boot<-c(Pvals.boot, pvals.boot)
}
)

##############################################################################
