#setwd("C:/Users/ythuang/Desktop/mediation_test3/R codes")
source("MT_Comp.R")

library(limma)	# propTrueNull()
type="mixnormal"	# "normal", "t", "uniform", or "mixnormal"


###########################
## Step 1: Simulate data ##
###########################

n0<-2000
n1<-3000
n2<-5000
n3<-0
n<-n0+n1+n2+n3
sda<-1
sdb<-1
set.seed(37)
if (type=="normal"){
	mua<-rnorm(n1, mean=0, sd=sda)
	mub<-rnorm(n2, mean=0, sd=sdb)
	main="Normal"
}
if (type=="t"){
	mua<-rt(n1, ncp=0, df=5)*sqrt(3/5)*sda
	mub<-rt(n2, ncp=0, df=5)*sqrt(3/5)*sdb
	main="t distribution"
}
if (type=="uniform"){
	mua<-runif(n1, min=-1, max=1)*sqrt(3)*sda
	mub<-runif(n2, min=-1, max=1)*sqrt(3)*sda
	main="Uniform"
}
if (type=="mixnormal"){
	rba<-rbinom(n1, 1, 0.3) 
	rbb<-rbinom(n1, 1, 0.4)
	mua<-rba*rnorm(n1, mean=0, sd=0.5)+(1-rba)*rnorm(n1, mean=0, sd=1.5)
	mub<-rbb*rnorm(n1, mean=0, sd=0.5)+(1-rbb)*rnorm(n1, mean=0, sd=1.5)
	mua<-sda*mua/sd(mua)
	mub<-sdb*mub/sd(mub)
	main="Mixture of normal"
}

a0<-rnorm(n0, mean=0)
a1<-rnorm(n1, mean=mua)
a2<-rnorm(n2, mean=0)
a3<-rnorm(n3, mean=mua*1)
b0<-rnorm(n0, mean=0)
b1<-rnorm(n1, mean=0)
b2<-rnorm(n2, mean=mub)
b3<-rnorm(n3, mean=mub*1)

a<-c(a0, a1, a2, a3)
b<-c(b0, b1, b2, b3)


###########################
## Step 2: Perform tests ##
###########################

## Our proposed method
pp.hat.comp<-MT_Comp(a, b)

## Joint significance test
pa<-2*pnorm(abs(a), low=F)
pb<-2*pnorm(abs(b), low=F)
pp.jt<-apply(cbind(pa, pb), 1, max)

## Using the proportion of null (theoretical)
ab<-a*b
pp0<-sapply(abs(ab)/sqrt(1), myp)
pp1.star<-sapply(abs(ab)/sqrt(1+sda^2), myp)
pp2.star<-sapply(abs(ab)/sqrt(1+sdb^2), myp)
w0.star<-n0/n; w1.star<-n1/n; w2.star<-n2/n
pp.star.comp<-(w0.star*pp0+w1.star*pp1.star+w2.star*pp2.star)

## Using the proportion of null (estimated)
p1.hat<-propTrueNull(pa)
p2.hat<-propTrueNull(pb)
w0<-p1.hat*p2.hat; w1<-(1-p2.hat)*p1.hat; w2<-(1-p1.hat)*p2.hat
w<-w0+w1+w2; w0<-w0/w; w1<-w1/w; w2<-w2/w
pp.tilde.comp<-w0*pp0+w1*pa+w2*pb


###############################
## Step 3: Summarize results ##
###############################

## Histogram
par(mfrow=c(2,2))
hist(pp.star.comp)
hist(pp.hat.comp)
hist(pp.tilde.comp)
hist(pp.jt)

## No (%) with cut-offs
pp<-cbind(pp.star.comp, pp.hat.comp, pp.tilde.comp, pp.jt)
apply(pp<0.1, 2, mean)*100
apply(pp<0.05, 2, mean)*100
apply(pp<0.01, 2, mean)*100
apply(pp<0.005, 2, mean)*100
apply(pp<0.001, 2, mean)*100

## QQ plot
par(mfrow=c(1,1), mar=c(4,4,3,2))
pt.sort<-sort(runif(length(pp.jt)))
pp.jt.sort<-sort(pp.jt)
plot(y=-log10(pp.jt.sort), x=-log10(pt.sort), ylim=c(0, 4), xlim=c(0,4), col=4, pch=20,
	ylab=expression(paste("Observed ", -log[10]*italic(p))), xlab=expression(paste("Theoretical ", -log[10]*italic(p))),
	main=main)
legend("topleft", 
	legend=c(expression(italic(p[comp])), expression(italic(tilde(p)[comp])), expression(italic(hat(p)[comp])), expression(italic(p)[JT])), col=1:4, bty="n", pch=16)
abline(0, 1)
for (j in 1:3){
	pp.sort<-sort(pp[,j])
	points(y=-log10(pp.sort), x=-log10(pt.sort), col=c(1,3,2)[j], pch=20)
}

